//==============================================================================
//
// Projet  : FPROFIL
//           Classe de fichier de profil et fonctions d'acces
//
// Version : 1.00  Juillet 1999
// Auteur  : Denis ZIEGLER  denis.ziegler@worldonline.fr
//
// Fichier : FPROFIL.H
//           Fichier inclus de FPROFIL.CPP
//
// Portabilite : C++ ANSI (DOS, Windows, Unix)
//
// (C) Copyright 1999 - ZIEGLER Denis
// Diffusion/modification autorisee selon les termes de la licence d'utilisation
//==============================================================================

#include "Fich2.h"


//===== Fichiers de profil =====================================================
//
//  Fichiers utilises par les applications pour enregistrer des parametres.
//  Ce sont des fichiers texte ASCII editables par tout editeur de texte.
//  Ils sont concus sur le modele des fichiers *.INI des applications Windows.
//  Chaque fichier comporte un certain nombre d'entrees sous la forme :
//    <identifiant>=<valeur>
//  Les entrees sont groupees par sections avec un titre de la forme :
//    [<section>]
//  Des lignes de commentaires, commencant par ; peuvent etre ajoutees.
//
//  Exemple :   ;--- Fichier exemple TEST.INI -------
//              [Affichage]
//              ecran=SVGA
//              mode=graphique
//
//              [Fichiers]
//              repertoire de travail=c:\donnees\test\


//===== Classe de fichier de profil ============================================
//
//  Elle controle l'acces en lecture ou ecriture aux fichiers de profil.
//  Le profil est charge automatiquement en memoire lors de la construction de
//  l'objet, ou manuellement a la demande.
//  Les operations de lecture/ecriture sont tres rapides car realisees sur la
//  copie du profil en memoire.
//  Le fichier de profil est mis a jour automatiquement lors de la destruction
//  de l'objet, ou manuellement a la demande.

class FichProf : public FichTxt {
 public :
  //--- Fonctions membres publiques --------------------------------------------
  FichProf(char *nomfich, int _lectauto=1, int _ecrauto=1);  //constructeur
    //Constructeur
    //  parametres :
    //    nomfich  : nom du fichier, avec le chemin d'acces.
    //    _lectauto : indicateur de lecture automatique du profil a l'ouverture
    //    _ecrauto  : indicateur d'ecriture automatique du profil a la fermeture
  ~FichProf(void);
    //Destructeur
  int LireCh(char *sect, char *entr, char *defaut, char *dest, int dim);
    //Lecture d'une entree dans le profil
    //  parametres :
    //    sect   : identifiant de la section a consulter
    //    entr   : identifiant de l'entree a consulter (si vide copie de toute la section)
    //    defaut : chaine renvoyee par defaut, si non trouve.
    //    dest   : chaine destination du resultat
    //    dim    : dimension maxi pour dest (nb de car. y compris zero terminal)
    //  valeur retournee : nb de caracteres lus (sans le zero terminal)
  int LireInt(char *sect, char *entr, int defaut);
    //Lecture d'une entree sous forme d'un nombre entier dans le profil
    //  parametres :
    //    sect   : identifiant de la section a consulter
    //    entr   : identifiant de l'entree a consulter
    //    defaut : valeur renvoyee par defaut, si non trouve.
    //  valeur retournee : entier lu, zero si erreur de conversion
  int EcrireDonnees(char *sect, char *entr, char *val);
    //Ecriture dans le profil
    //  parametres :
    //    sect   : identifiant de la section a ecrire
    //    entr   : identifiant de l'entree a ecrire
    //    val    : valeur a ecrire (chaine)
    //    Si l'entree existe, la valeur est remplacee,
    //    sinon une nouvelle entree est cree.
    //    Si val = NULL, l'entree est supprimee
    //    Si entr = NULL, toute la section est supprimee
    //  valeur retournee : 0 = erreur   <>0 = Ok
  int LireProfil(void);
    //Lecture du profil sur disque et stockage en memoire
  int EcrireProfil(void);
    //Ecriture du profil sur disque (remplace fichier precedent ou cree nouveau)
  int Modif(void) {return modif;}
    // Indique si le profil a ete modifie depuis la dernire operation de
    // lecture/ecriture
 protected :
  //--- Structures de donnees privees ------------------------------------------
  // Structures de donnees d'une entree et d'une section
  //   Les differentes sections du profil sont chainees dans une liste lineaire
  //   Chaque section comprend des entrees chainees de la meme facon
  struct Sentree {
    int comment;       //indique si la ligne est un commentaire (valeur <> 0)
    char *id;          //identifiant de l'entree
    char *val;         //valeur (texte) de l'entree
    Sentree *suivt;    //pointeur vers l'entree suivante
  };
  struct Ssection {
    int comment;       //indique si la ligne est un commentaire (valeur <> 0)
    char *id;          //identifiant de la section
    Sentree *entr;     //pointeur vers les entrees de la section
    Ssection *suivt;   //pointeur vers la section suivante
  };
  //--- Donnees membres privees ------------------------------------------------
  Ssection *profil;    //pointeur vers le profil
  Ssection *sectcour;  //pointeur vers la section courante
  Sentree *entrcour;   //pointeur vers l'entree courante
  int modif;           //indicateur de modification du profil
  int lectauto;           //indicateur de lecture automatique du profil
  int ecrauto;           //indicateur d'ecriture automatique du profil
  //--- Fonctions membres privees (cf code source) -----------------------------
  int LireLgn(void);
  int LireSection(char *ch);
  int LireEntree(char *ch);
  int LireCommentaire(char *ch);
  int EcrireSection(void);
  int EcrireEntree(void);
  void MajProfil(char *sect, char *entr, char *val);
  void MajSection(Ssection* sect, char *entr, char *val);
  void MajEntree(Sentree* entr, char *val);
  Ssection *NouvelleSection(char *sect, char *entr, char *val);
  Sentree *NouvelleEntree(char *entr, char *val);
  void AjouteCh(char *dest, char *source, int &i, int imax);
  void DesallouerProfil(void);
  void DesallouerSection(Ssection *sect);
  void DesallouerEntree(Sentree *entr);
};


//===== Fonctions d'acces aux fichiers de profil ===============================
//
//  Ces fonctions sont calquees sur celles de l'API Windows.
//  Elles permettent la lecture/ecriture dans les fichiers de profil.
//  Elles ne sont pas aussi performantes que l'utilisation directe de la classe
//  FichProf, car chaque appel de fonction entraine l'ouverture, la lecture,
//  eventuellement l'ecriture, puis la fermeture du fichier de profil.

int WritePrivateProfileString(char *section, char *entree, char *source,
                              char *nomfich);
  //Ecriture dans le profil
  //  parametres :
  //    section : identifiant de la section a consulter
  //    entree  : identifiant de l'entree a consulter
  //    source  : valeur a ecrire (chaine)
  //    nomfich : nom du fichier de profil (avec chemin d'acces)
  //    Si l'entree existe, la valeur est remplacee,
  //    sinon une nouvelle entree est cree.
  //    Si source = NULL, l'entree est supprimee
  //    Si entree = NULL, toute la section est supprimee
  //  valeur retournee : 0 = erreur   <>0 = Ok
int GetPrivateProfileString(char *section, char *entree, char *defaut,
                            char *dest, int tdest, char *nomfich);
  //Lecture dans le profil
  //  parametres :
  //    section : identifiant de la section a consulter
  //    entree  : identifiant de l'entree a consulter
  //    defaut  : valeur par defaut
  //    dest    : chaine destination
  //    tdest   : taille maxi de dest (y compris zero terminal)
  //    nomfich : nom du fichier de profil (avec chemin d'acces)
  //  valeur retournee : nb de caracteres lus (sans le zero terminal)
int GetPrivateProfileInt(char *section, char *entree, int defaut, char *nomfich);
  //Lecture dans le profil d'une valeur numerique entiere
  //  parametres :
  //    section : identifiant de la section a consulter
  //    entree  : identifiant de l'entree a consulter
  //    defaut  : valeur par defaut
  //    nomfich : nom du fichier de profil (avec chemin d'acces)
  //  valeur retournee : entier lu, zero si erreur de conversion



//=== Routines diverses ========================================================

//----- Comparaison de deux chaines de caracteres sans distinction min/maj -----
//      Equivalent a stricmp, non disponible en C ANSI.
int stricomp(const char *ch1, const char *ch2);

